/*
 * Copyright (c) 2017, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.wso2.carbon.device.mgt.ios.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.device.mgt.ios.core.bean.ChallengeTokenBean;
import org.wso2.carbon.device.mgt.ios.core.exception.IOSEnrollmentException;
import org.wso2.carbon.device.mgt.mobile.impl.ios.enrollment.IOSEnrollmentTokenService;

public class EnrollmentUtils {
    private static final Log log = LogFactory.getLog(EnrollmentUtils.class);
    private static IOSEnrollmentTokenService tokenService = null;

    static {
        log.info("Initializing IOSEnrollmentTokenService.");
        try {
            tokenService = IOSServiceUtils.getEnrollmentTokenService();
        } catch (IOSEnrollmentException e) {
            log.error("IOSEnrollmentTokenService cannot be initialized.", e);
        }
    }

    /**
     * Retrieve challenge token entry.
     *
     * @param challengeToken challenge token
     * @return {@link ChallengeTokenBean}
     * @throws IOSEnrollmentException if unable to retrieve challenge token entry.
     */
    public static ChallengeTokenBean getTokenEntry(String challengeToken) throws IOSEnrollmentException {
        if (tokenService == null) {
            tokenService = IOSServiceUtils.getEnrollmentTokenService();
        }
        return tokenService.getDeviceEnrollmentToken(challengeToken);
    }
}